local _,ns = ...
local oUF = oUF or ns.oUF
local healcomm = LibStub("LibHealComm-4.0")

local mediapath = 'Interface\\AddOns\\oUF_Taroven\\media\\'

local height = 18
local shortheight = 12
local powerheight = 4
local width = 170
local shortwidth = 100
local tinywidth = width - shortwidth - 3
local raidpadding = 4
local raidpower = 2
local raidaurabar = 2

local aurasize = 18
local auraspacing = 2
local iconsize = 14
local fontsize = 10
local largefontsize = 12

local hpcolor = {0,0,0,0.2}
local hpbgcolor = {0,0,0,0.2}
local border = mediapath..'border'
local texture = mediapath..'TaroF'
local bgtex = mediapath..'TaroF'
local bgalpha = 0.6
local bgahigh = 0.7
local glowtex = mediapath..'glowTex'
local ptex = mediapath..'TaroAura2'
local font = mediapath..'font.ttf'

local gsub = string.gsub
local format = string.format
local floor = math.floor

local localized,class = UnitClass('player')
local raidsize = (width-(raidpadding*4))/5		-- (width - (padding * #pads))/#raidmembers

local bossList = {
	['The Lich King'] = {10,40,70},
	['Sindragosa'] = {35,85},
	['Professor Putricide'] = {35,80},
	['Deathbringer Saurfang'] = {30},
}

local classPercs = {
	['WARRIOR'] = {20},
	['HUNTER'] = {20},
	['PALADIN'] = {20},
}

local auraList = {	-- ['SpellName'] = {AuraFunction (UnitBuff or UnitDebuff),{colors R,G,B,[A]},Position (0 = lowest),caster (unitID,'any',or blank for player)}
	['DRUID'] = {
		['Lifebloom'] = {UnitBuff,{1,0.5,1},2},
		['Rejuvenation'] = {UnitBuff,{0,1,1},1},
		['Regrowth'] = {UnitBuff,{0,1,0},0},
		['Wild Growth'] = {UnitBuff,{0,1,0,1},-1},
	},
	['PRIEST'] = {
		['Weakened Soul'] = {UnitDebuff,{1,0,0},0,'any'},
		['Renew'] = {UnitBuff,{0,1,0},1},
		['Grace'] = {UnitBuff,{0,1,1},-1},
	},
	['SHAMAN'] = {
		['Riptide'] = {UnitBuff,{0,1,0},0},
	},
	['PALADIN'] = {
		['Sacred Shield'] = {UnitBuff,{1,1,0},0},
		['Flash of Light'] = {UnitBuff,{0,1,0},1},
		['Beacon of Light'] = {UnitBuff,{0,1,1},-1},
	},
}

-- Spells to be shown in the Stacks field
local stackSpells = {
	['DRUID'] = GetSpellInfo(33763),	-- Lifebloom
	['PRIEST'] = GetSpellInfo(33076),	-- Prayer of Mending
	['SHAMAN'] = GetSpellInfo(974),		-- Earth Shield
	['WARRIOR'] = GetSpellInfo(50720),	-- Vigilance
}

-- Colors per stack amount
local stackColors = {
	['DRUID'] = {	-- Lifebloom
		{1,0,0},
		{1,1,0},
		{0,1,0},
	},
	['PRIEST'] = {	-- Prayer of Mending
		{1,0,0},
		{1,1,0},
		{1,1,0},
		{0,1,0},
		{0,1,0},
	},
	['SHAMAN'] = {	-- Earth Shield
		{1,0,0},
		{1,0,0},
		{1,1,0},
		{1,1,0},
		{0,1,0},
		{0,1,0},
		{0,1,0},
		{0,1,0},
	},
}

local CanDispel = {
	PRIEST = { Magic = true,Disease = true,},
	SHAMAN = { Poison = true,Disease = true,},
	PALADIN = { Magic = true,Poison = true,Disease = true,},
	MAGE = { Curse = true,},
	DRUID = { Curse = true,Poison = true,}
}
if not CanDispel[class] then CanDispel[class] = {} end

local backdrop = {
	bgFile = [=[Interface\ChatFrame\ChatFrameBackground]=],
	insets = {top = 2,left = 2,bottom = 2,right = 2},
}

local menu = function (self)
	local unit = gsub(self.unit,'(.)',string.upper,1)
	if(_G[unit..'FrameDropDown']) then
		ToggleDropDownMenu(1,nil,_G[unit..'FrameDropDown'],'cursor')
	end
end

local truncate = function (value)
	if(value >= 1e6) then
		return gsub(format('%.2fm',value / 1e6),'%.?0+([km])$','%1')
	elseif(value >= 1e4) then
		return gsub(format('%.1fk',value / 1e3),'%.?0+([km])$','%1')
	else
		return value
	end
end

local htruncate = function (value)
	if(value >= 1e6) then
		return gsub(format('%.2fm',value / 1e6),'%.?0+([km])$','%1')
	elseif(value >= 1e4) then
		return gsub(format('%.1fk',value / 1e3),'%.?0+([km])$','%1')
	else
		return '+'..value
	end
end

local shorten = function (name)
	local i,j = string.find(name,'....')
	if i then
		return string.sub(name,i,j)
	else
		return name
	end
end

local Round = function (num,idp)
  if idp and idp>0 then
    local mult = 10^idp
    return floor(num * mult + 0.5) / mult
  end
  return floor(num + 0.5)
end

local CalcAuras = function (framewidth,aurasize)
	local numauras = framewidth/aurasize
	local padding = (framewidth-(aurasize*numauras))/numauras
	
	if padding < 1 then
		while padding < 1 do
			numauras = numauras-1
			padding = (framewidth-(aurasize*numauras-1))/numauras
		end
	end
	return numauras,padding
end

local hex = function (r,g,b)
	local r,g,b = r,g,b
	if(type(r) == 'table') then
		if(r.r) then r,g,b = r.r,r.g,r.b else r,g,b = unpack(r) end
	end
	if not r then r,g,b = .5,.5,.5 end
	return string.format('|cff%02x%02x%02x',r * 255,g * 255,b * 255)
end

local createAura = function (self,button)
	button.parent.showDebuffType = true
	button.cd:SetReverse()
	
	button.count:SetFont(font,11,'OUTLINE')
	button.count:SetPoint('BOTTOMRIGHT',button,'BOTTOMRIGHT',-2,2)

	button.overlay:SetTexture(border)
	button.overlay:SetTexCoord(0,1,0,1)
	button.overlay.Hide = function(self) self:SetVertexColor(1,1,1) end
end

local Hide = function (self)
	if self.HCB then 
		self.HCB:SetValue(0)
		self.HCB:Hide() end
	if self.HCB2 then
		self.HCB2:Hide()
		self.HCB2:SetValue(0)
	end
	if self.HCT then self.HCT:SetText("") end
end

-- Borrowed from Freebaser - Usage: string, beginning letter, length (UnitName, 1, 4)
local function utf8sub(str, start, numChars) 
	local currentIndex = start 
	while numChars > 0 and currentIndex <= #str do 
	    local char = string.byte(str, currentIndex) 
	    if char >= 240 then 
	      currentIndex = currentIndex + 4 
	    elseif char >= 225 then 
	      currentIndex = currentIndex + 3 
	    elseif char >= 192 then 
	      currentIndex = currentIndex + 2 
	    else 
	      currentIndex = currentIndex + 1 
	    end 
		numChars = numChars - 1 
	end 
	return str:sub(start, currentIndex - 1) 
end 

local auraFunc = function (u)
	local name,rank,icon,count,debuffType,duration,expirationTime,source = UnitBuff(u,stackSpells[class])
	if name and count and (source == 'player') then
		return hex(stackColors[class][count])..count
	end
end

local EarthShield
local auraFuncUnique = function (u)
	if class == 'SHAMAN' and not(EarthShield) then return end
	local name,rank,icon,count,debuffType,duration,expirationTime,source = UnitBuff(u,stackSpells[class])
	if name and count then
		if source == 'player' then
			return hex(stackColors[class][count])..count
		else
			return hex(.5,.5,.5)..count
		end
	end
end

local tags = {
	['T:namelong'] = {
		event = 'UNIT_HEALTH',
		func = function (u)
			local r,g,b,t
			local min,max = UnitHealth(u),UnitHealthMax(u)
			local isdead = UnitIsDeadOrGhost(u)
			
			if isdead or not UnitIsConnected(u) then
				r,g,b = .6,.6,.6
			elseif(UnitIsPlayer(u)) then
				local _,class = UnitClass(u)
				t = oUF.colors.class[class]
			else
				r,g,b = .2,.5,.5
			end

			if(t) then
				r,g,b = t[1],t[2],t[3]
			end
			local per = min/max

			if (per > .95) or isdead then
				return hex(r,g,b)..UnitName(u)
			else
				return hex(r,g,b)..'-'..truncate(max-min)
			end
		end,
	},

	['T:nameshort'] = {
		event = 'UNIT_HEALTH',
		func = function (u)
			local r,g,b,t
			local min,max = UnitHealth(u),UnitHealthMax(u)
			local isdead = UnitIsDeadOrGhost(u)
			
			if isdead or not UnitIsConnected(u) then
				r,g,b = .6,.6,.6
			elseif(UnitIsPlayer(u)) then
				local _,class = UnitClass(u)
				t = oUF.colors.class[class]
			else
				r,g,b = .2,.5,.5
			end

			if(t) then
				r,g,b = t[1],t[2],t[3]
			end
			local per = min/max

			if (per > .95) or isdead then
				return hex(r,g,b)..utf8sub(UnitName(u),1,4)
			else
				return hex(r,g,b)..'-'..truncate(max-min)
			end
		end,
	},

	['PRIEST'] = {
		event = 'UNIT_AURA',
		func = auraFunc,
	},
	
	['DRUID'] = {
		event = 'UNIT_AURA',
		func = auraFunc,
	},
	
	['SHAMAN'] = {
		event = 'UNIT_AURA',
		func = auraFuncUnique,
	},
	
	['WARRIOR'] = {
		event = 'UNIT_AURA',
		func = function (u)
			local name,rank,icon,count,debuffType,duration,expirationTime,source = UnitBuff(u,stackSpells.WARRIOR)
			if name and (source == 'player') then
				return hex(0,1,0)..'v'
			elseif name then
				return hex(.5,.5,.5)..'v'
			end
		end,
	},
}

for k,v in pairs(tags) do
	if k == 'T:nameshort' or k == 'T:namelong' or k == class then
		oUF.Tags[k] = v.func
		oUF.TagEvents[k] = v.event
	end
end

local UpdateHealComm = function (self,event)
	if not self.unit or UnitIsDeadOrGhost(self.unit) or not UnitIsConnected(self.unit) then return Hide(self) end
	local maxHP = UnitHealthMax(self.unit) or 0
	if maxHP == 0 or maxHP == 100 then return Hide(self) end
	local guid = UnitGUID(self.unit)
	local overTime = bit.bor(healcomm.HOT_HEALS, healcomm.BOMB_HEALS)
	local timeFrame = self.HCTimeframe and GetTime() + self.HCTimeframe or nil
	local incHeals = healcomm:GetHealAmount(guid, healcomm.CASTED_HEALS, timeFrame) or 0
	local hotHeals = healcomm:GetHealAmount(guid, overTime, timeFrame) or 0
	
	if hotHeals == 0 and incHeals == 0 then	return Hide(self) end

	incHeals = incHeals * healcomm:GetHealModifier(guid)

	if self.HCT then
		if incHeals and incHeals > 0 then
			self.HCT:SetText(self.HCTFormat and self.HCTFormat(incHeals) or format("%d", incHeals))
		else
			self.HCT:SetText("")
		end
	end

	local curHP = UnitHealth(self.unit)
	if self.HCB then	-- Direct heals
		local percInc = math.min(incHeals, maxHP - curHP) / maxHP
		if percInc == 0 then
			self.HCB:SetValue(0)
			self.HCB:Hide()
		else
			self.HCB:SetValue( (math.min(incHeals, maxHP - curHP) / maxHP) / (1 - curHP/maxHP) )	-- This basically scales the healcomm bar inverse to the unit's missing health. Strange concept, but it allows dynamic anchoring, which is exactly what we're shooting for.
			self.HCB:Show()
		end
	end
	if self.HCB2 then	-- HoT effects + lifebloom bomb
		local percInc = math.min(hotHeals, maxHP - curHP) / maxHP
		if percInc == 0 then
			self.HCB2:SetValue(0)
			self.HCB2:Hide()
		else
			self.HCB2:SetValue( (math.min(hotHeals, maxHP - curHP) / maxHP) / (1 - curHP/maxHP) )
			self.HCB2:Show()
		end
	end		
end

local updateHealth = function(self,unit,min,max)
	if not unit then return end
	local r,g,b,t
	local isdead = UnitIsDeadOrGhost(unit)
	
	if isdead or not UnitIsConnected(unit) then
		r,g,b = .6,.6,.6
	elseif(UnitIsPlayer(unit)) then
		local _,class = UnitClass(unit)
		t = oUF.colors.class[class]
	else
		r,g,b = .2,.5,.5
	end

	if(t) then
		r,g,b = t[1],t[2],t[3]
	end

	self.bg:SetVertexColor(r,g,b,0.5)
end

local f = CreateFrame('Frame')
f:SetScript('OnEvent',function(self,event,...)
	return self[event](self,...)
end)

local EventsToRegister = {
	['PLAYER_TARGET_CHANGED'] = true,
	['PLAYER_FOCUS_CHANGED'] = true,
	['UNIT_AURA'] = true,
	['PLAYER_ALIVE'] = true,
	['PLAYER_ENTERING_WORLD'] = true,
	['ACTIVE_TALENT_GROUP_CHANGED'] = true,
	['PLAYER_TALENT_UPDATE'] = true,
}

f.UNIT_AURA = function (self,unit)
	local frame = oUF.units[unit]
	if not frame or frame.unit ~= unit then return end	
	
	if not frame.auraBars then return end
	local ab = frame.auraBars
	for k,v in pairs(ab) do
		local name,rank,icon,count,debuffType,duration,expirationTime,source = v.AuraFunction(unit,k)
		if name and (source == v.Caster or v.Caster == 'any') then
			v.stop = expirationTime
			v.start = expirationTime - duration
			v.duration = duration
			v:SetScript('OnUpdate',function(self,elapsed)
				self.current = (self.stop - GetTime())/duration
				self:SetValue(self.current)
				if self.current < 0 then self:SetScript('OnUpdate',nil) end
			end)
		else
			v.current = 0
			v:SetValue(0)
			v:SetScript('OnUpdate',nil)
		end
	end
end

f.PLAYER_TALENT_UPDATE = function ()
	if not(GetTalentInfo(1,1)) then return end
	if class == 'SHAMAN' then
		local cs = select(5,GetTalentInfo(3,18)) or 0
		local es = select(5,GetTalentInfo(3,26)) or 0
		if cs > 0 then
			CanDispel.SHAMAN.Curse = true
		else
			CanDispel.SHAMAN.Curse = nil
		end
		if es > 0 then EarthShield = true else EarthShield = nil end
	end
end

-- Aura bars
f.PLAYER_TARGET_CHANGED = function(self) f.UNIT_AURA(self,'target') end
f.PLAYER_FOCUS_CHANGED = function(self) f.UNIT_AURA(self,'focus') end
-- Talent checks
f.PLAYER_ALIVE = f.PLAYER_TALENT_UPDATE
f.PLAYER_ENTERING_WORLD = f.PLAYER_TALENT_UPDATE
f.ACTIVE_TALENT_GROUP_CHANGED = f.PLAYER_TALENT_UPDATE
-- Register applicable events
for k,v in pairs(EventsToRegister) do
	if v then
		f:RegisterEvent(k)
	end
end

local function GetDebuffType(unit)
	if not UnitCanAssist('player',unit) then return nil end
	local i = 1
	while true do
		local name,rank,texture,count,debufftype,duration,expirationTime,source = UnitDebuff(unit,i)
		if not texture then break end
		if debufftype and CanDispel[class][debufftype] then
			return debufftype
		end
		i = i + 1
	end
end

local function updateSituation(self,event,u)
	if not u or self.unit ~= u then return end
	local debufftype = GetDebuffType(u)
	local s = UnitThreatSituation(u)
	if debufftype then
		local color = DebuffTypeColor[debufftype] 
		self.Glow:SetBackdropBorderColor(color.r,color.g,color.b,bgahigh)
		self.Health:SetStatusBarColor(color.r,color.g,color.b,0.9)
	elseif s and s > 1 then
		r,g,b = GetThreatStatusColor(s)
		self.Glow:SetBackdropBorderColor(r,g,b,bgahigh)
		self.Health:SetStatusBarColor(0,0,0,0.65)
	else
		self.Glow:SetBackdropBorderColor(0,0,0,bgalpha)
		self.Health:SetStatusBarColor(0,0,0,0.65)
	end
end

local menu = function(self)
	local unit = self.unit:gsub('(.)',string.upper,1) 
	if _G[unit..'FrameDropDown'] then
		ToggleDropDownMenu(1,nil,_G[unit..'FrameDropDown'],'cursor')
	elseif (self.unit:match('party')) then
		ToggleDropDownMenu(1,nil,_G['PartyMemberFrame'..self.id..'DropDown'],'cursor')
	else
		FriendsDropDown.unit = self.unit
		FriendsDropDown.id = self.id
		FriendsDropDown.initialize = RaidFrameDropDown_Initialize
		ToggleDropDownMenu(1,nil,FriendsDropDown,'cursor')
	end
end

local UnitSpecific = {
	player = function (self)
		-- Standard settings
		self:SetAttribute('initial-height',height)
		self:SetAttribute('initial-width',width)
		
		self.Power:SetPoint('BOTTOMRIGHT',self)
		self.Power:SetPoint('BOTTOMLEFT',self)
		self.Power:SetHeight(powerheight)
		
		self.Health:SetPoint('TOPRIGHT',self)
		self.Health:SetPoint('TOPLEFT',self)
		self.Health:SetPoint('BOTTOM',self.Power,'TOP')
		
		self.Health.frequentUpdates = true
		self.Power.frequentUpdates = true
		
		-- Castbar
		self.Castbar = CreateFrame('StatusBar',nil,self)
		self.Castbar:SetFrameStrata('HIGH')
		self.Castbar:SetStatusBarTexture(texture)

		self.Castbar:SetAllPoints(self.Health)
		self.Castbar:SetStatusBarColor(1,1,1)
		self.Castbar:SetAlpha(0.5)
		
		-- Extra tags
		self.Health.perc = self.Health:CreateFontString(nil,'OVERLAY')
		self.Health.perc:SetFont(font,largefontsize)
		self.Health.perc:SetShadowColor(0,0,0,0.5)
		self.Health.perc:SetShadowOffset(2,-2)
		self.Health.perc:SetAlpha(0.5)
		self.Health.perc:SetPoint('BOTTOMLEFT',self.Health,'RIGHT',1,0)
		self:Tag(self.Health.perc,'[raidcolor][perhp]')
		
		self.Health.val = self.Health:CreateFontString(nil,'OVERLAY')
		self.Health.val:SetFont(font,largefontsize)
		self.Health.val:SetShadowColor(0,0,0,0.5)
		self.Health.val:SetShadowOffset(2,-2)
		self.Health.val:SetAlpha(0.5)
		self.Health.val:SetPoint('TOPLEFT',self.Health,'RIGHT',1,0)
		self:Tag(self.Health.val,'[raidcolor][curhp]')
		
		self.Castbar.Text = self.Castbar:CreateFontString(nil,'OVERLAY')
		self.Castbar.Text:SetAllPoints(self.Castbar)
		self.Castbar.Text:SetFont(font,fontsize)
		self.Castbar.Text:SetShadowColor(0,0,0,0.5)
		self.Castbar.Text:SetShadowOffset(1,-1)
		self.Castbar.Text:SetAlpha(0.9)
		
		-- Icons
		self.Leader = self.Health:CreateTexture(nil,'OVERLAY')
		self.Leader:SetPoint('BOTTOMLEFT',self.Health,'BOTTOMLEFT',height/2 - 1,0)
		self.Leader:SetHeight(height/2 - 1)
		self.Leader:SetWidth(height/2 - 1)
		self.Leader:SetAlpha(0.4)
		
		self.RaidIcon:SetHeight(14)
		self.RaidIcon:SetWidth(14)
		
		self.LFDRole = self.Health:CreateTexture(nil,'OVERLAY')
		self.LFDRole:SetPoint('BOTTOMLEFT',self.Health,'TOPRIGHT',-3,0)
		self.LFDRole:SetWidth(height/2 - 1)
		self.LFDRole:SetHeight(height/2 - 1)		
		self.LFDRole:SetAlpha(0.4)
		
		-- Healcomm
		self.HCB = CreateFrame('StatusBar',nil,self.Health)
		self.HCB:SetMinMaxValues(0,1)
		self.HCB:SetStatusBarTexture(texture)
		self.HCB:SetStatusBarColor(0.25,1,0,0.5)
		self.HCB:SetPoint("BOTTOMLEFT", self.Health:GetStatusBarTexture(), "BOTTOMRIGHT")
		self.HCB:SetPoint("TOPRIGHT", self.Health, "TOPRIGHT")
		self.HCB:SetOrientation("HORIZONTAL")
		
		self.HCB2 = CreateFrame('StatusBar',nil,self.Health)
		self.HCB2:SetMinMaxValues(0,1)
		self.HCB2:SetStatusBarTexture(texture)
		self.HCB2:SetStatusBarColor(0,0.5,0.75,0.5)
		self.HCB2:SetPoint("BOTTOMLEFT", self.Health:GetStatusBarTexture(), "BOTTOMRIGHT")
		self.HCB2:SetPoint("TOPRIGHT", self.Health, "TOPRIGHT")
		self.HCB2:SetOrientation("HORIZONTAL")
		
		self.HCTimeframe = 2.5
		
		local numauras,padding = CalcAuras(width,aurasize)
		self.Debuffs = CreateFrame('Frame',nil,self)
		self.Debuffs:SetPoint('BOTTOM',self.Health,'TOP',0,1)
		self.Debuffs:SetHeight(aurasize)
		self.Debuffs:SetWidth(width)
		self.Debuffs.size = aurasize
		self.Debuffs.spacing = padding
		self.Debuffs.num = numauras
		self.Debuffs.initialAnchor = 'TOPRIGHT'
		self.Debuffs['growth-x'] = 'LEFT'
		
		self.Debuffs.PostCreateIcon = createAura
		
		self.Stacks = self.Health:CreateFontString(nil,'ARTWORK')
		if tags[class] then self:Tag(self.Stacks,'['..class..']') end
		self.Stacks:SetPoint('LEFT',self,'LEFT')
		self.Stacks:SetFont(font,fontsize+1,'OUTLINE')
	end,

	pet = function (self)
		self:SetAttribute('initial-height',height)
		self:SetAttribute('initial-width',tinywidth)
		
		self.Power:SetPoint('BOTTOMRIGHT',self)
		self.Power:SetPoint('BOTTOMLEFT',self)
		self.Power:SetHeight(powerheight)
		self.Power.frequentUpdates = true
		
		self.Health:SetPoint('TOPRIGHT',self)
		self.Health:SetPoint('TOPLEFT',self)
		self.Health:SetPoint('BOTTOM',self.Power,'TOP')
		
		self.Castbar = CreateFrame('StatusBar',nil,self)
		self.Castbar:SetFrameStrata('HIGH')
		self.Castbar:SetStatusBarTexture(texture)

		self.Castbar:SetAllPoints(self.Health)
		self.Castbar:SetStatusBarColor(1,1,1)
		self.Castbar:SetAlpha(0.5)
		
		self.RaidIcon:SetHeight(8)
		self.RaidIcon:SetWidth(8)
		
		local numauras,padding = CalcAuras(tinywidth,aurasize-2)
		self.Buffs = CreateFrame('Frame',nil,self)
		self.Buffs:SetWidth(tinywidth)
		self.Buffs.size = aurasize-2
		self.Buffs.spacing = padding
		self.Buffs:SetPoint("BOTTOMRIGHT",self,"TOPRIGHT",0,1)
		self.Buffs:SetHeight((aurasize * 4) + (padding * 3))
		self.Buffs.num = numauras*4
		self.Buffs.initialAnchor = "BOTTOMRIGHT"
		self.Buffs['growth-x'] = 'LEFT'
		self.Buffs['growth-y'] = 'UP'
		self.Buffs.onlyShowPlayer = true
		
		self.Buffs.PostCreateIcon = createAura
		
		self.Stacks = self.Health:CreateFontString(nil,'ARTWORK')
		if tags[class] then self:Tag(self.Stacks,'['..class..']') end
		self.Stacks:SetPoint('LEFT',self,'LEFT')
		self.Stacks:SetFont(font,fontsize+1,'OUTLINE')
	end,
	
	target = function (self)
		self:SetAttribute('initial-height',height)
		self:SetAttribute('initial-width',width)
		
		self.Power:SetPoint('BOTTOMRIGHT',self)
		self.Power:SetPoint('BOTTOMLEFT',self)
		self.Power:SetHeight(powerheight)
		
		self.Health:SetPoint('TOPRIGHT',self)
		self.Health:SetPoint('TOPLEFT',self)
		self.Health:SetPoint('BOTTOM',self.Power,'TOP')
		
		self.Castbar = CreateFrame('StatusBar',nil,self)
		self.Castbar:SetFrameStrata('HIGH')
		self.Castbar:SetStatusBarTexture(texture)

		self.Castbar:SetAllPoints(self.Health)
		self.Castbar:SetStatusBarColor(1,1,1)
		self.Castbar:SetAlpha(0.8)
		
		-- Extra tags
		self.Health.perc = self.Health:CreateFontString(nil,'OVERLAY')
		self.Health.perc:SetFont(font,largefontsize)
		self.Health.perc:SetShadowColor(0,0,0,0.5)
		self.Health.perc:SetShadowOffset(2,-2)
		self.Health.perc:SetAlpha(0.5)
		self.Health.perc:SetPoint('BOTTOMRIGHT',self.Health,'LEFT',-1,0)
		self:Tag(self.Health.perc,'[raidcolor][perhp]')
		
		self.Health.val = self.Health:CreateFontString(nil,'OVERLAY')
		self.Health.val:SetFont(font,largefontsize)
		self.Health.val:SetShadowColor(0,0,0,0.5)
		self.Health.val:SetShadowOffset(2,-2)
		self.Health.val:SetAlpha(0.5)
		self.Health.val:SetPoint('TOPRIGHT',self.Health,'LEFT',-1,0)
		self:Tag(self.Health.val,'[raidcolor][curhp]')
		
		self.Castbar.Text = self.Castbar:CreateFontString(nil,'OVERLAY')
		self.Castbar.Text:SetAllPoints(self.Castbar)
		self.Castbar.Text:SetFont(font,fontsize)
		self.Castbar.Text:SetShadowColor(0,0,0,0.5)
		self.Castbar.Text:SetShadowOffset(1,-1)
		self.Castbar.Text:SetAlpha(0.9)
		
		self.RaidIcon:SetHeight(14)
		self.RaidIcon:SetWidth(14)
		
		-- Healcomm
		self.HCB = CreateFrame('StatusBar',nil,self.Health)
		self.HCB:SetMinMaxValues(0,1)
		self.HCB:SetStatusBarTexture(texture)
		self.HCB:SetStatusBarColor(0.25,1,0,0.5)
		self.HCB:SetPoint("BOTTOMLEFT", self.Health:GetStatusBarTexture(), "BOTTOMRIGHT")
		self.HCB:SetPoint("TOPRIGHT", self.Health, "TOPRIGHT")
		self.HCB:SetOrientation("HORIZONTAL")
		
		self.HCB2 = CreateFrame('StatusBar',nil,self.Health)
		self.HCB2:SetMinMaxValues(0,1)
		self.HCB2:SetStatusBarTexture(texture)
		self.HCB2:SetStatusBarColor(0,0.5,0.75,0.5)
		self.HCB2:SetPoint("BOTTOMLEFT", self.Health:GetStatusBarTexture(), "BOTTOMRIGHT")
		self.HCB2:SetPoint("TOPRIGHT", self.Health, "TOPRIGHT")
		self.HCB2:SetOrientation("HORIZONTAL")
		
		self.HCTimeframe = 2.5
		
		local numauras,padding = CalcAuras(focus and shortwidth or width,aurasize)
		self.Debuffs = CreateFrame('Frame',nil,self)
		self.Debuffs:SetPoint('BOTTOM',self.Health,'TOP',0,1)
		self.Debuffs:SetHeight(aurasize)
		self.Debuffs:SetWidth(focus and shortwidth or width)
		self.Debuffs.size = aurasize
		self.Debuffs.spacing = padding
		self.Debuffs.num = numauras
		self.Debuffs.initialAnchor = 'TOPLEFT'
		self.Debuffs['growth-x'] = 'RIGHT'
		self.Debuffs.PostCreateIcon = createAura
				
		self.Buffs = CreateFrame('Frame',nil,self)
		self.Buffs:SetWidth(width)
		self.Buffs.size = aurasize
		self.Buffs.spacing = padding
		self.Buffs:SetPoint('TOPLEFT',self.Health,'TOPRIGHT',self.Buffs.spacing,0)
		self.Buffs:SetHeight((aurasize * 4) + (padding * 3))
		self.Buffs.num = numauras*4
		self.Buffs.initialAnchor = 'TOPLEFT'
		self.Buffs['growth-x'] = 'RIGHT'
		self.Buffs['growth-y'] = 'DOWN'
		self.Buffs.PostCreateIcon = createAura
		
		self.Phases = CreateFrame('Frame',nil,self.Health)
		self.Phases:SetAllPoints(self.Health)
		self.Phases.tex = {}
		self.Phases.fs = {}
		
		local effectiveScale = self.Health:GetEffectiveScale()
		local onepixelwide = 1
		if effectiveScale then
			onepixelwide = 1/effectiveScale
		end
		
		for i = 1,5 do
			local tex = self.Phases:CreateTexture(nil,'OVERLAY')
			local fs = self.Phases:CreateFontString(nil,'OVERLAY')
			tex:SetWidth(onepixelwide)
			tex:SetTexture(1,1,1)
			tex:SetVertexColor(1,1,1,0.8)
			tex:SetPoint('TOP',self.Phases)
			tex:SetAlpha(0)
			
			fs:SetFont(font,fontsize)
			fs:SetShadowColor(0,0,0,0.5)
			fs:SetShadowOffset(1,-1)
			fs:SetVertexColor(1,1,1,0.6)
			
			self.Phases.tex[i] = tex
			self.Phases.fs[i] = fs
		end
		
		self.Phases:RegisterEvent('PLAYER_TARGET_CHANGED')
		self.Phases:RegisterEvent('PLAYER_REGEN_ENABLED')
		self.Phases:RegisterEvent('PLAYER_REGEN_DISABLED')
		
		self.Phases:SetScript('OnEvent', function(self)
			local unitname = UnitName('target')
			local name = UnitExists('target') and unitname and bossList[unitname]
			if name then
				for i = 1,5 do
					if name[i] then
						local t = self.tex[i]
						t:SetPoint('BOTTOMLEFT', self,'BOTTOMLEFT', width * (name[i]/100), 0)
						self.fs[i]:SetText(name[i])
						self.fs[i]:SetPoint('TOPRIGHT',t,'TOP',-3,0)
						t:SetAlpha(1)
					else
						self.tex[i]:SetAlpha(0)
						self.fs[i]:SetText("")
					end
				end
			else
				for i = 1,5 do self.tex[i]:SetAlpha(0); self.fs[i]:SetText("") end
			end
		end)
	end,
	
	targettarget = function (self)
		self:SetAttribute('initial-height',shortheight)
		self:SetAttribute('initial-width',shortwidth)
		
		self.Power:SetPoint('BOTTOMRIGHT',self)
		self.Power:SetPoint('BOTTOMLEFT',self)
		self.Power:SetHeight(powerheight)
		
		self.Health:SetPoint('TOPRIGHT',self)
		self.Health:SetPoint('TOPLEFT',self)
		self.Health:SetPoint('BOTTOM',self.Power,'TOP')
		
		self.RaidIcon:SetHeight(8)
		self.RaidIcon:SetWidth(8)
		
		-- Name
		self.Name = self.Health:CreateFontString(nil,'OVERLAY')
		self:Tag(self.Name,'[raidcolor][name]')
		self.Name:SetFont(font,fontsize-2)
		self.Name:SetAllPoints(self.Health)
		self.Name:SetShadowColor(0,0,0,0.5)
		self.Name:SetShadowOffset(1,-1)
		
		self:SetAttribute('initial-height',shortheight)
		self:SetAttribute('initial-width',shortwidth)
		self.Health:SetHeight(shortheight)
		self.Health:SetWidth(shortwidth)
		
		local numauras,padding = CalcAuras(shortwidth,aurasize-2)
		self.Debuffs = CreateFrame('Frame',nil,self)
		self.Debuffs:SetPoint('TOPRIGHT',self.Health,'TOPLEFT',1,0)
		self.Debuffs:SetHeight(aurasize-2)
		self.Debuffs:SetWidth(shortwidth)
		self.Debuffs.size = aurasize-2
		self.Debuffs.spacing = padding
		self.Debuffs.num = numauras
		self.Debuffs.initialAnchor = 'TOPRIGHT'
		self.Debuffs['growth-x'] = 'LEFT'
		self.Debuffs['growth-y'] = 'DOWN'
		self.Debuffs.PostCreateIcon = createAura
	end,
	
	targettargettarget = function (self)		
		self:SetAttribute('initial-height',shortheight)
		self:SetAttribute('initial-width',tinywidth)
		self.Health:SetHeight(shortheight)
		self.Health:SetWidth(tinywidth)
		
		self.Power:SetPoint('BOTTOMRIGHT',self)
		self.Power:SetPoint('BOTTOMLEFT',self)
		self.Power:SetHeight(powerheight)
		self.Power.frequentUpdates = true
		
		self.Health:SetPoint('TOPRIGHT',self)
		self.Health:SetPoint('TOPLEFT',self)
		self.Health:SetPoint('BOTTOM',self.Power,'TOP')
		
		self.RaidIcon:SetHeight(8)
		self.RaidIcon:SetWidth(8)
		
		-- Name
		self.Name = self.Health:CreateFontString(nil,'OVERLAY')
		self:Tag(self.Name,'[raidcolor][name]')
		self.Name:SetFont(font,fontsize-2)
		self.Name:SetAllPoints(self.Health)
		self.Name:SetShadowColor(0,0,0,0.5)
		self.Name:SetShadowOffset(1,-1)
	end,
	
	focus = function (self)
		self:SetAttribute('initial-height',shortheight)
		self:SetAttribute('initial-width',shortwidth)
		
		self.Power:SetPoint('BOTTOMRIGHT',self)
		self.Power:SetPoint('BOTTOMLEFT',self)
		self.Power:SetHeight(powerheight)
		self.Power.frequentUpdates = true
		
		self.Health:SetPoint('TOPRIGHT',self)
		self.Health:SetPoint('TOPLEFT',self)
		self.Health:SetPoint('BOTTOM',self.Power,'TOP')
		
		-- Name
		self.Name = self.Health:CreateFontString(nil,'OVERLAY')
		self:Tag(self.Name,'[raidcolor][name]')
		self.Name:SetFont(font,fontsize-2)
		self.Name:SetAllPoints(self.Health)
		self.Name:SetShadowColor(0,0,0,0.5)
		self.Name:SetShadowOffset(1,-1)
		
		self.Castbar = CreateFrame('StatusBar',nil,self)
		self.Castbar:SetFrameStrata('HIGH')
		self.Castbar:SetStatusBarTexture(texture)

		self.Castbar:SetAllPoints(self.Health)
		self.Castbar:SetStatusBarColor(1,1,1)
		self.Castbar:SetAlpha(0.5)
		
		self.RaidIcon:SetHeight(8)
		self.RaidIcon:SetWidth(8)
		
		-- Healcomm
		self.HCB = CreateFrame('StatusBar',nil,self.Health)
		self.HCB:SetMinMaxValues(0,1)
		self.HCB:SetStatusBarTexture(texture)
		self.HCB:SetStatusBarColor(0.25,1,0,0.5)
		self.HCB:SetPoint("BOTTOMLEFT", self.Health:GetStatusBarTexture(), "BOTTOMRIGHT")
		self.HCB:SetPoint("TOPRIGHT", self.Health, "TOPRIGHT")
		self.HCB:SetOrientation("HORIZONTAL")
		self.HCB2 = CreateFrame('StatusBar',nil,self.Health)
		self.HCB2:SetMinMaxValues(0,1)
		self.HCB2:SetStatusBarTexture(texture)
		self.HCB2:SetStatusBarColor(0,0.5,0.75,0.5)
		self.HCB2:SetPoint("BOTTOMLEFT", self.Health:GetStatusBarTexture(), "BOTTOMRIGHT")
		self.HCB2:SetPoint("TOPRIGHT", self.Health, "TOPRIGHT")
		self.HCB2:SetOrientation("HORIZONTAL")
		
		self.HCTimeframe = 2.5
		
		self.SpellRange = true
		self.inRangeAlpha = 1.0
		self.outsideRangeAlpha = 0.4
		
		local numauras,padding = CalcAuras(shortwidth,aurasize-2)
		self.Debuffs = CreateFrame('Frame',nil,self)
		self.Debuffs:SetPoint('TOPLEFT',self.Health,'TOPRIGHT',1,0)
		self.Debuffs:SetHeight(aurasize-2)
		self.Debuffs:SetWidth(shortwidth)
		self.Debuffs.size = aurasize-2
		self.Debuffs.spacing = padding
		self.Debuffs.num = numauras
		self.Debuffs.initialAnchor = 'TOPLEFT'
		self.Debuffs['growth-x'] = 'RIGHT'
		self.Debuffs['growth-y'] = 'DOWN'
		self.Debuffs.PostCreateIcon = createAura
		
		self.Stacks = self.Health:CreateFontString(nil,'ARTWORK')
		if tags[class] then self:Tag(self.Stacks,'['..class..']') end
		self.Stacks:SetPoint('LEFT',self,'LEFT')
		self.Stacks:SetFont(font,fontsize+1,'OUTLINE')
	end,
	
	focustarget = function (self)	
		self:SetAttribute('initial-height',shortheight)
		self:SetAttribute('initial-width',tinywidth)
		
		self.Health:SetHeight(shortheight)
		self.Health:SetWidth(tinywidth)
		
		self.Power:SetPoint('BOTTOMRIGHT',self)
		self.Power:SetPoint('BOTTOMLEFT',self)
		self.Power:SetHeight(powerheight)
		self.Power.frequentUpdates = true
		
		self.Health:SetPoint('TOPRIGHT',self)
		self.Health:SetPoint('TOPLEFT',self)
		self.Health:SetPoint('BOTTOM',self.Power,'TOP')
		
		self.RaidIcon:SetHeight(8)
		self.RaidIcon:SetWidth(8)
		
		-- Name
		self.Name = self.Health:CreateFontString(nil,'OVERLAY')
		self:Tag(self.Name,'[raidcolor][name]')
		self.Name:SetFont(font,fontsize-2)
		self.Name:SetAllPoints(self.Health)
		self.Name:SetShadowColor(0,0,0,0.5)
		self.Name:SetShadowOffset(1,-1)
	end,
	
	-- Party frames, the neverending thorn in my side, and arguably the prettiest of the bunch.
	party = function (self)		
		self:SetAttribute('initial-height',height)
		self:SetAttribute('initial-width',shortwidth)
		
		self.Power:SetPoint('BOTTOMRIGHT',self)
		self.Power:SetPoint('BOTTOMLEFT',self)
		self.Power:SetHeight(powerheight)
		
		self.Health:SetPoint('TOPRIGHT',self)
		self.Health:SetPoint('TOPLEFT',self)
		self.Health:SetPoint('BOTTOM',self.Power,'TOP')
		self.Health.frequentUpdates = true
		
		self.Castbar = CreateFrame('StatusBar',nil,self)
		self.Castbar:SetFrameStrata('HIGH')
		self.Castbar:SetStatusBarTexture(texture)

		self.Castbar:SetAllPoints(self.Health)
		self.Castbar:SetStatusBarColor(1,1,1)
		self.Castbar:SetAlpha(0.3)
		
		-- Name
		self.Name = self.Health:CreateFontString(nil,'OVERLAY')
		self:Tag(self.Name,'[T:namelong]')
		self.Name:SetFont(font,fontsize-2)
		self.Name:SetAllPoints(self.Health)
		self.Name:SetShadowColor(0,0,0,0.5)
		self.Name:SetShadowOffset(1,-1)
		
		-- Leader icon
		self.Leader = self.Health:CreateTexture(nil,'OVERLAY')
		self.Leader:SetPoint(raid and 'CENTER' or 'BOTTOMLEFT',self.Health,raid and 'TOPLEFT' or 'BOTTOMLEFT',raid and 0 or height/2 - 1,0)
		self.Leader:SetHeight((raid or party) and 8 or height/2 - 1)
		self.Leader:SetWidth((raid or party) and 8 or height/2 - 1)
		self.Leader:SetAlpha(0.4)
		
		self.RaidIcon:SetHeight(14)
		self.RaidIcon:SetWidth(14)
		
		-- Healcomm
		self.HCB = CreateFrame('StatusBar',nil,self.Health)
		self.HCB:SetMinMaxValues(0,1)
		self.HCB:SetStatusBarTexture(texture)
		self.HCB:SetStatusBarColor(0.25,1,0,0.5)
		self.HCB:SetPoint("BOTTOMLEFT", self.Health:GetStatusBarTexture(), "BOTTOMRIGHT")
		self.HCB:SetPoint("TOPRIGHT", self.Health, "TOPRIGHT")
		self.HCB:SetOrientation("HORIZONTAL")
		self.HCB2 = CreateFrame('StatusBar',nil,self.Health)
		self.HCB2:SetMinMaxValues(0,1)
		self.HCB2:SetStatusBarTexture(texture)
		self.HCB2:SetStatusBarColor(0,0.5,0.75,0.5)
		self.HCB2:SetPoint("BOTTOMLEFT", self.Health:GetStatusBarTexture(), "BOTTOMRIGHT")
		self.HCB2:SetPoint("TOPRIGHT", self.Health, "TOPRIGHT")
		self.HCB2:SetOrientation("HORIZONTAL")
		
		self.HCTimeframe = 2.5
		
		-- Buffs
		local numbuffs,padding = CalcAuras(width,aurasize)
		self.Buffs = CreateFrame('Frame',nil,self)
		self.Buffs:SetHeight(height)
		self.Buffs:SetWidth(width)
		self.Buffs.size = height
		self.Buffs.spacing = padding
		self.Buffs.num = numbuffs
		self.Buffs:SetPoint('BOTTOMRIGHT',self,'BOTTOMLEFT',-5,0)
		self.Buffs.initialAnchor = 'BOTTOMRIGHT'
		self.Buffs['growth-x'] = 'LEFT'
		self.Buffs['growth-y'] = 'DOWN'
		self.Buffs.onlyShowPlayer = true
		self.Buffs.filter = 'HELPFUL|RAID'
		self.Buffs.PostCreateIcon = createAura
		
		-- Debuffs
		local numdebuffs,dpadding = CalcAuras(shortwidth,height*0.65)
		self.Debuffs = CreateFrame('Frame',nil,self)
		self.Debuffs:SetHeight(height*0.65)
		self.Debuffs:SetWidth(shortwidth)
		self.Debuffs.size = height*0.65
		self.Debuffs.spacing = dpadding
		self.Debuffs.num = numdebuffs
		self.Debuffs:SetPoint('LEFT',self,'TOPLEFT',0,2)
		self.Debuffs.initialAnchor = 'TOPLEFT'
		self.Debuffs['growth-x'] = 'RIGHT'
		self.Debuffs:SetAlpha(0.6)
		self.Debuffs.PostCreateIcon = createAura
		
		-- LFD role icons
		self.LFDRole = self.Health:CreateTexture(nil,'OVERLAY')
		self.LFDRole:SetPoint('BOTTOMLEFT',self.Health,'BOTTOMRIGHT')
		self.LFDRole:SetWidth(height/2 - 1)
		self.LFDRole:SetHeight(height/2 - 1)		
		self.LFDRole:SetAlpha(0.4)
		
		-- Readycheck
		self.ReadyCheck = self.Health:CreateTexture(nil,'OVERLAY')
		self.ReadyCheck:SetPoint('RIGHT',self.Health,'RIGHT')
		self.ReadyCheck:SetWidth(10)
		self.ReadyCheck:SetHeight(10)
		
		self.SpellRange = true
		self.inRangeAlpha = 1.0
		self.outsideRangeAlpha = 0.4
		
		self.Stacks = self.Health:CreateFontString(nil,'ARTWORK')
		if tags[class] then self:Tag(self.Stacks,'['..class..']') end
		self.Stacks:SetPoint('LEFT',self,'LEFT')
		self.Stacks:SetFont(font,fontsize+1,'OUTLINE')
	end,
	
	raid = function (self)
		--self.Health.frequentUpdates = true	-- Testing without for a while. Better performance iz gud.		
		
		self:SetAttribute('initial-height',raidsize)
		self:SetAttribute('initial-width',raidsize)
		self.Health:SetOrientation('VERTICAL')
		self.Power:SetOrientation('VERTICAL')
		
		self.Power:SetOrientation('VERTICAL')
		self.Power:SetPoint('TOPRIGHT',self)
		self.Power:SetPoint('BOTTOMRIGHT',self)
		self.Power:SetWidth(raidpower)
		
		self.Health:SetPoint('TOPLEFT',self)
		self.Health:SetPoint('BOTTOMRIGHT',self.Power,"BOTTOMLEFT")
		
		-- Name
		self.Name = self.Health:CreateFontString(nil,'OVERLAY')
		self:Tag(self.Name,'[T:nameshort]')
		self.Name:SetFont(font,fontsize-2)
		self.Name:SetPoint("BOTTOM",self.Health,"CENTER")
		self.Name:SetPoint("TOP",self.Health,"TOP")
		self.Name:SetShadowColor(0,0,0,0.5)
		self.Name:SetShadowOffset(1,-1)
		
		-- Leader icon
		self.Leader = self.Health:CreateTexture(nil,'OVERLAY')
		self.Leader:SetPoint('CENTER',self.Health,'TOPLEFT')
		self.Leader:SetHeight(8)
		self.Leader:SetWidth(8)
		self.Leader:SetAlpha(0.4)
		
		self.RaidIcon:SetHeight(8)
		self.RaidIcon:SetWidth(8)
		
		-- Healcomm
		self.HCB = CreateFrame('StatusBar',nil,self.Health)
		self.HCB:SetMinMaxValues(0,1)
		self.HCB:SetStatusBarTexture(texture)
		self.HCB:SetStatusBarColor(0.25,1,0,0.5)
		self.HCB:SetPoint("BOTTOMLEFT", self.Health:GetStatusBarTexture(), "TOPLEFT")
		self.HCB:SetPoint("TOPRIGHT", self.Health, "TOPRIGHT")
		self.HCB:SetOrientation("VERTICAL")
		
		self.HCB2 = CreateFrame('StatusBar',nil,self.Health)
		self.HCB2:SetMinMaxValues(0,1)
		self.HCB2:SetStatusBarTexture(texture)
		self.HCB2:SetStatusBarColor(0,0.5,0.75,0.5)
		self.HCB2:SetPoint("BOTTOMLEFT", self.Health:GetStatusBarTexture(), "TOPLEFT")
		self.HCB2:SetPoint("TOPRIGHT", self.Health, "TOPRIGHT")
		self.HCB2:SetOrientation("VERTICAL")
		
		self.HCTimeframe = 2.5
		
		--Heal Text
		local heal = self.Health:CreateFontString(nil,'OVERLAY')
		heal:SetPoint('BOTTOM')
		heal:SetJustifyH('CENTER')
		heal:SetFont(font,fontsize-2)
		heal:SetTextColor(0,1,0,1)
		heal:SetShadowColor(0,0,0,0.5)
		heal:SetShadowOffset(1,-1)
		heal:SetAlpha(0.8)
		
		self.HCT = heal
		self.HCTFormat = htruncate
		self.HCT:SetAllPoints(self.Health)
		
		self.ReadyCheck = self.Health:CreateTexture(nil,'OVERLAY')
		self.ReadyCheck:SetPoint('BOTTOM',self.Health,'BOTTOM')
		self.ReadyCheck:SetWidth(10)
		self.ReadyCheck:SetHeight(10)
		
		self.SpellRange = true
		self.inRangeAlpha = 1.0
		self.outsideRangeAlpha = 0.4
		
		self.Stacks = self.Health:CreateFontString(nil,'ARTWORK')
		if tags[class] then self:Tag(self.Stacks,'['..class..']') end
		self.Stacks:SetPoint('BOTTOMLEFT',self,'BOTTOMLEFT')
		self.Stacks:SetFont(font,fontsize+1,'OUTLINE')
		
		self.auraBars = {}
		if auraList[class] then
			local i = 0
			for k,v in pairs(auraList[class]) do
				self.auraBars[k] = CreateFrame('StatusBar',nil,self.Health)
				local b = self.auraBars[k]
				b:SetMinMaxValues(0,1)
				b:SetValue(0)
				if v[3] >= 0 then
					b:SetPoint('BOTTOM',self.Health,'BOTTOM',0,(v[3]*raidaurabar) + (v[3]*2))
				else
					b:SetPoint('TOP',self.Health,'TOP',0,(v[3] + 1) + ((v[3] + 1)*2))
				end
				b:SetPoint('LEFT',self.Health)
				b:SetPoint('RIGHT',self.Health)
				b:SetHeight(raidaurabar)
				b:SetWidth(self.Health:GetWidth())
				b:SetAlpha(v[2][4] and v[2][4] or 0.8)
				b.AuraFunction = v[1]
				b:SetStatusBarTexture(texture)
				b:SetStatusBarColor(v[2][1],v[2][2],v[2][3])
				b.Caster = v[4] or 'player'
				i = i + 1
			end
		end
	end,
}

local function styleFunction(self,unit)
	local focus = unit == 'focus'
	local pet = unit == 'pet'
	local player = unit == 'player'
	local target = unit == 'target'
	local tot = unit == 'targettarget'
	local totot = unit == 'targettargettarget'
	local tof = unit == 'focustarget'
	local party = self:GetParent():GetName():match('oUF_Taroven_party')
	local raid = self:GetParent():GetName():match('oUF_Taroven_raid')

	self.menu = menu
	self:RegisterForClicks('AnyUp')
	self:SetAttribute('type2','menu')

	self:SetScript('OnEnter',UnitFrame_OnEnter)
	self:SetScript('OnLeave',UnitFrame_OnLeave)
	
	self.Glow = CreateFrame('Frame',nil,self)
	self.Glow:SetPoint('TOPLEFT',self,'TOPLEFT',-4,4)
	self.Glow:SetPoint('BOTTOMRIGHT',self,'BOTTOMRIGHT',4,-4)
	self.Glow:SetFrameStrata('BACKGROUND')
	self.Glow:SetBackdrop {
		edgeFile = glowtex,edgeSize = 5,
		edgeFile = glowtex,edgeSize = 5,
		insets = {left = 3,right = 3,top = 3,bottom = 3}
	}
	self.Glow:SetBackdropColor(0,0,0,0)
	self.Glow:SetBackdropBorderColor(0,0,0,bgalpha)
	
	self.Health = CreateFrame('StatusBar',self:GetName()..'_health',self)
	self.Health:SetStatusBarTexture(texture)
	self.Health:SetStatusBarColor(0,0,0,0.65)
	--self.Health:GetStatusBarTexture():SetVertTile(false)
	--self.Health:GetStatusBarTexture():SetHorizTile(false)
	self.Glow.bar = self.Health
	
	self.Power = CreateFrame('StatusBar',self:GetName()..'_power',self)
	self.Power:SetStatusBarTexture(ptex)
	--self.Power:GetStatusBarTexture():SetVertTile(false)
	--self.Power:GetStatusBarTexture():SetHorizTile(false)
	
	self.Health.bg = self.Health:CreateTexture(nil,'BORDER')
	self.Health.bg:SetAllPoints(self.Health)
	self.Health.bg:SetTexture(bgtex)
	
	self.Power.bg = self.Power:CreateTexture(nil,'BORDER')
	self.Power.bg:SetAllPoints(self.Power)
	self.Power.bg:SetTexture(ptex)
	self.Power.bg.multiplier = 0.3
	--self.Power.bg:SetAlpha(0.5)
	
	self.Power.colorTapping = true
	self.Power.colorDisconnected = true
	self.Power.colorClass = true
	self.Power.colorPower = pet
	self.Power.colorHappiness = pet
	self.Power.colorReaction = not pet
	
	self.RaidIcon = self.Health:CreateTexture(nil,'OVERLAY')
	self.RaidIcon:SetPoint('CENTER',self,'TOP',0,0)
	
	self.Health.PostUpdate = updateHealth
	
	self:RegisterEvent('UNIT_THREAT_LIST_UPDATE',updateSituation)
	self:RegisterEvent('UNIT_THREAT_SITUATION_UPDATE',updateSituation)
	self:RegisterEvent('UNIT_AURA',updateSituation)
	self:RegisterEvent('PLAYER_REGEN_ENABLED',updateSituation)
	self:RegisterEvent('PLAYER_REGEN_DISABLED',updateSituation)
	self:RegisterEvent("UNIT_HEALTH", UpdateHealComm)
	self:RegisterEvent("UNIT_MAXHEALTH", UpdateHealComm)
	
	if UnitSpecific[unit] then
		return UnitSpecific[unit](self)
	elseif party then
		return UnitSpecific.party(self)
	elseif raid then
		return UnitSpecific.raid(self)
	end
end

oUF:RegisterStyle('Taroven',styleFunction)
for unit,layout in next, UnitSpecific do
	oUF:RegisterStyle('Taroven - ' .. unit:gsub("^%l", string.upper), styleFunction)
end

local spawnHelper = function (self, unit, ...)
	if(UnitSpecific[unit]) then
		self:SetActiveStyle('Taroven - ' .. unit:gsub("^%l", string.upper))
		local object = self:Spawn(unit)
		object:SetPoint(...)
		return object
	else
		self:SetActiveStyle'Taroven'
		local object = self:Spawn(unit)
		object:SetPoint(...)
		return object
	end
end

local raid = {}
local party
oUF:Factory(function(self)
	local player = spawnHelper(self,'player','TOPRIGHT',UIParent,'CENTER',-100,-130)
	local target = spawnHelper(self,'target','TOPLEFT',UIParent,'CENTER',100,-130)
	local focus = spawnHelper(self,'focus','TOPRIGHT',player,'BOTTOMRIGHT',0,-3)
	local tot = spawnHelper(self,'targettarget','TOPLEFT',target,'BOTTOMLEFT',0,-3)
	spawnHelper(self,'focustarget','RIGHT',focus,'LEFT',-3,0)
	spawnHelper(self,'targettargettarget','LEFT',tot,'RIGHT',3,0)
	spawnHelper(self,'pet','RIGHT',player,'LEFT',-3,0)
	
	self:SetActiveStyle('Taroven - Party')
	party = self:SpawnHeader('oUF_Taroven_party',nil,'custom [group:party,nogroup:raid][@raid6,noexists,group:raid] show;hide',
		'showParty',true,
--		'showRaid',true,		-- May need this if raid members <= 5
--		'showSolo',true,		-- Debug/test
--		'showPlayer',true,		-- Debug/test
		'point','TOP',
		'xOffset',0,
		'yOffset',-height*0.35)
	party:SetPoint('TOPRIGHT',focus,'BOTTOMRIGHT',0,-height*0.35)
	
	self:SetActiveStyle('Taroven - Raid')
	for i = 1,5 do
		local raidg = self:SpawnHeader('oUF_Taroven_raid'..i,nil,'raid10',
			'groupFilter',tostring(i),
			'showRaid',true,
--			'showSolo',true,		-- Debug/test
			'showParty',true,		-- Needs moar testing
			'showPlayer',true,		-- And moar.
			'point','RIGHT',
			'xoffset',-raidpadding,
			'yOffset',0)
		table.insert(raid,raidg)
		if(i == 1) then	
			raidg:SetPoint('TOPRIGHT',focus,'BOTTOMRIGHT',0,-5)
		else
			raidg:SetPoint('TOPRIGHT',raid[i-1],'BOTTOMRIGHT',0,-raidpadding) 
		end
	end
		
end)

local function MultiUpdate(...)
	for i = 1, select("#", ...) do
		for _, frame in ipairs(oUF.objects) do
			if frame.unit and (frame.HCB or frame.HCT) and UnitGUID(frame.unit) == select(i, ...) then UpdateHealComm(frame) end
		end
	end
end

local function HealComm_Heal_Update(event, casterGUID, spellID, healType, _, ...)
	MultiUpdate(...)
end

local function HealComm_Modified(event, guid)
	MultiUpdate(guid)
end

healcomm.RegisterCallback("oUF_HealComm4", "HealComm_HealStarted", HealComm_Heal_Update)
healcomm.RegisterCallback("oUF_HealComm4", "HealComm_HealUpdated", HealComm_Heal_Update)
healcomm.RegisterCallback("oUF_HealComm4", "HealComm_HealDelayed", HealComm_Heal_Update)
healcomm.RegisterCallback("oUF_HealComm4", "HealComm_HealStopped", HealComm_Heal_Update)
healcomm.RegisterCallback("oUF_HealComm4", "HealComm_ModifierChanged", HealComm_Modified)
healcomm.RegisterCallback("oUF_HealComm4", "HealComm_GUIDDisappeared", HealComm_Modified)

